"""
.. codeauthor:: Nathan Hertz <nhertz@nrao.edu>

File containing definitions for the other half of the capability side of the Workspaces REST API,
concerning capability requests
"""
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPPreconditionFailed
from pyramid.request import Request
from pyramid.response import Response
from pyramid.view import view_config

from workspaces.capability.schema import CapabilityRequest


@view_config(route_name="view_capability_request", renderer="json")
def view_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to view a capability request and responds with the request's info, if it exists
    URL: capability/{capability_name}/request/{request_id}

    :param request: GET request
    :return: Response with JSON-formatted capability request info if found
        or 404 response (HTTPNotFound)
    """
    capability_request = request.capability_info.lookup_capability_request(
        request.matchdict["request_id"]
    )
    if capability_request:
        return Response(json_body=capability_request.__json__())
    else:
        not_found_msg = (
            f"Capability request for capability {request.matchdict['capability_name']}",
            f"with ID {request.matchdict['request_id']} not found.",
        )
        return HTTPNotFound(detail=not_found_msg)


@view_config(route_name="create_capability_request", renderer="json")
def create_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to create a capability request
    URL: capability/{capability_name}/request/create

    :param request: POST request, expecting JSON parameters ["parameters", "versions"]
    :return: Response with JSON-formatted info of newly created capability request
        or 400 response (HTTPBadRequest) if expected parameters not given
        or 412 response (HTTPPreconditionFailed) if capability with given name does not exist and thus cannot be
        requested
    """
    expected_params = ["parameters"]
    capability_name = request.matchdict["capability_name"]
    params = request.json_body

    if not all([expected in params for expected in expected_params]):
        # JSON params do not contain all expected params
        params_not_given_msg = (
            f"Expected JSON parameters {expected_params}. Received only {params}."
        )
        return HTTPBadRequest(detail=params_not_given_msg)
    elif not request.capability_info.lookup_capability(capability_name):
        # Capability with given name does not exist; can't create a request for it
        does_not_exist_msg = (
            f"Capability {capability_name} does not exist. Cannot create request.",
        )
        return HTTPPreconditionFailed(detail=does_not_exist_msg)
    else:
        # TODO: Implement future products
        new_capability_request = request.capabilities.create_request(
            capability_name, parameters=params["parameters"], products=None
        )
        return Response(json_body=new_capability_request.__json__())


@view_config(route_name="edit_capability_request", renderer="json")
def edit_capability_request(request: Request) -> Response:
    """
    Pyramid view that accepts a request to edit a capability request
    URL: capability/{capability_name}/request/{request_id}/edit

    TODO: Implement once CapabilityVersions are supported
    """
    return HTTPBadRequest()
